// T35preCat_OnEst.sce
// PREDICTION WITH DISCRETE MODEL (ON-LINE)
// Change: - length of prediction
//         - uncertainty of the simulated model
//         - imput signal
//         - study the beginning when estimation is not finished
//           how can we secure quicker transient phase of estimation?
// Remark: another way og generation is
//            y(t)=sum(rand(1,1,'u')>cumsum(th(i,:)))+1;
// -------------------------------------------------------------------
exec("ScIntro.sce",-1),mode(0)

nd=150;                            // number of data
np=2;                              // length of prediction
th1=[0.98 0.01 0.04 0.97]';        // parameters (for y=1)
th=[th1 1-th1];                    // all parameters
u=(rand(1,nd+np,'u')>.3)+1;        // input
y(1)=1;

// TIME LOOP
nu=1e-8*ones(4,2);
Et=zeros(4,nd-np);
for t=2:nd                         // time loop
  // prediction
  i=2*(u(t)-1)+y(t-1);             // row of the table
  yy=sum(rand(1,1,'u')>cumsum(th(i,:)))+1; // prediction generation
  for j=1:np
    i=2*(u(t+j)-1)+yy;             // row of the table
    yy=sum(rand(1,1,'u')>cumsum(th(i,:)))+1; // prediction generation
  end
  yp(t+np)=yy;                     // np-step predction

  // simulation
  i=2*(u(t)-1)+y(t-1);
  y(t)=sum(rand(1,1,'u')>cumsum(th(i,:)))+1;  // output
 
  // estimation
  i=2*(u(t)-1)+y(t-1);             // row of model matrix
  nu(i,y(t))=nu(i,y(t))+1;         // statistics update
  Eth=nu./(sum(nu,2)*ones(1,2));   // pt estimates
  Et(:,t)=Eth(:,1);
end

// Results
disp(' Simulated parameters')
disp(th)
disp(' Estimated parameters')
disp(Eth)

s=np+2:np+51;
set(scf(),'position',[100 100 1000 400])
subplot(121),plot(Et')
title('Evolution of estimated parameters')
set(gca(),"data_bounds",[0 nd-np+1 -.1 1.1])
subplot(122),plot(s,y(s),s,yp(s),'.:')
title('First 50 outputs and their prediction')
set(gca(),"data_bounds",[s(1) s($) .9 2.1])

s=np+2:nd;
Wrong=sum(y(s)~=yp(s))
From=nd-np
